package com.solutionsbuilder.rangeseekbarwrapper;

/**
 * Created by Claudio Imperiale on 29/03/2015.
 *
 * B4A wrapper around the RangeSeekBar widget project at https://code.google.com/p/range-seek-bar/
 * **************************************************************************************************************
 * Widget that lets users select a minimum and maximum value on a given numerical range.
 * The range value types can be one of Long, Double, Integer, Float, Short, Byte or BigDecimal.<br />
 * <br />
 * Improved {@link MotionEvent} handling for smoother use, anti-aliased painting for improved aesthetics.
 *
 * @param <T> The Number type of the range values. One of Long, Double, Integer, Float, Short, Byte or BigDecimal.
 *            <p/>
 *            <p/>
 *            <p/>
 *            https://code.google.com/p/range-seek-bar/
 *            <p/>
 *            Apache License
 *            <p/>
 *            <p/>
 * @author Stephan Tittel (stephan.tittel@kom.tu-darmstadt.de)
 * @author Peter Sinnott (psinnott@gmail.com)
 * @author Thomas Barrasso (tbarrasso@sevenplusandroid.org)
 * @author Alex Florescu (florescu@yahoo-inc.com)
 * @author Michael Keppler (bananeweizen@gmx.de)
 * @author Claudio Imperiale
 * ************************************************************************************************************
 */

import com.yahoo.mobile.client.android.util.RangeSeekBar;

import android.view.ViewGroup;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Pixel;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.ViewWrapper;

//@DependsOn(values = { "rangeseekbar" })
@Version(1.00f)
@ShortName("RangeSeekBar")
@BA.Events(values={"Change"})
public class RangeSeekBarWrapper<T extends Number> extends ViewWrapper<RangeSeekBar<T>> implements DesignerCustomView
{
    // Throw away variable to communicate between Initialize2() and _initialize()
    boolean singleThumb = false;

    /**
     * Initialize the RangeSeekBar widget with AbsoluteMinValue = 0 and AbsoluteMaxValue = 100.
     * @param EventName set to a non-null string to enable Change event.
     */
    public void Initialize(final BA ba, String EventName) {
        _initialize(ba, null, EventName);
    }

    /**
     * Initialize the widget as a SeekBar - i.e. with one thumb only.
     * Defaults: AbsoluteMinValue = 0 and AbsoluteMaxValue = 100.
     * @param eventName set to a non-null string to enable Change event.
     */
    public void InitializeAsSingleThumb(final BA ba, String eventName) {
        this.singleThumb = true;
        _initialize(ba, null, eventName);
    }

    /**
     * Initialize the RangeSeekBar widget with AbsoluteMinValue = min and AbsoluteMaxValue = max.
     *If singleThumb is set to true, then a simple SeekBar is initialized.
     *
     * @param min Absolute minimnum value
     * @param max Absolute maximum value
     * @param singleThumb set to true, a SeekBar is initialized, instead of a RangeSeekBar
     * @param eventName set to a non-null string to enable Change event.
     */
    public void Initialize2(final BA ba, T min, T max, boolean singleThumb, String eventName) {
        this.singleThumb = singleThumb;
        _initialize(ba, null, eventName);
        getObject().setRangeValues(min, max);
    }

    @Hide
    public void _initialize(final BA ba, Object activityClass, String eventName) {

        this.ba = ba;

        //Make sure to do this, otherwise default wrapper view methods will fail in B4A
        this.setObject(new RangeSeekBar<T>(ba.context));
        if (this.singleThumb) {
            getObject().setSingleThumb(true);
            //noinspection unchecked
            getObject().setSelectedMinValue((T) RangeSeekBar.DEFAULT_MINIMUM);
        }
        getObject().setNotifyWhileDragging(true);

        //Add listeners
        this.addListeners(eventName.toLowerCase(BA.cul));
    }

    void addListeners(final String eventName) {
        if (RangeSeekBarWrapper.this.ba.subExists(eventName + "_change")) {
            this.getObject().setOnRangeSeekBarChangeListener(new RangeSeekBar.OnRangeSeekBarChangeListener<T>() {
                @Override
                public void onRangeSeekBarValuesChanged(RangeSeekBar<?> bar, T minValue, T maxValue) {
                    ba.raiseEvent2(getObject(), true, eventName + "_change", true, (Object[]) null);
                }
            });
        }
    }

    /**
     * Get/Set Min Label text
     */
    public String getMinLabelText() {
        return this.getObject().getMinLabel();
    }

    public void setMinLabelText(String minLabel) {
        this.getObject().setMinLabel(minLabel);
    }

    /**
     * Get/Set Max Label text
     */
    public String getMaxLabelText() {
        return this.getObject().getMaxLabel();
    }

    public void setMaxLabelText(String maxLabel) {
        this.getObject().setMaxLabel(maxLabel);
    }

    /**
     * Get: Return true if the RangeSeekBar has only one thumb. Default is false.
     * Set: Toggles between SeekBar and RangeSeekBar.
     */
    public boolean getSingleThumb() {
        return getObject().getSingleThumb();
    }

    public void setSingleThumb(boolean flag) {
        getObject().setSingleThumb(flag);
    }

    /**
     * Get/Set Text Color. Default is White
     */
    public int getTextColor()
    {
        return getObject().getTextColor();
    }

    public void setTextColor(int textColor) {
        getObject().setTextColor(textColor);
    }

    /**
     * Get/Set Label and Line Color. Default is White
     */
    public int getLabelColor()
    {
        return getObject().getLabelColor();
    }

    public void setLabelColor(int mLabelAndLineColor) {
        getObject().setLabelColor(mLabelAndLineColor);
    }

    /**
     * Toggle the display of the number above the RangeSeekBar's thumb.
     * @return boolean
     */
    public boolean getDrawNumber(){
        return this.getObject().getDrawText();
    }

    public void setDrawNumber(boolean flag) {
        this.getObject().setDrawText(flag);
    }
    /**
     * Set the Absolute Min and Max values after initialization.
     * The type can be either byte, integer, long, float or double.
     * @param minValue Minimum value.
     * @param maxValue Minimum value.
     */
    public void setRangeValues(T minValue, T maxValue) {
        getObject().setRangeValues(minValue, maxValue);
    }

    /**
     * Resets the currently selected values to the absolute min and max values.
     */
    public void resetSelectedValues() {
        getObject().resetSelectedValues();
    }

    public boolean getNotifyWhileDragging() {
        return getObject().isNotifyWhileDragging();
    }

    /**
     * Get: When true, the widget notify the listener callback while the user is still dragging a thumb.
     * Default is false.
     * Set: Enable/Disable change tracking.
     */
    public void setNotifyWhileDragging(boolean flag) {
        getObject().setNotifyWhileDragging(flag);
    }

    /**
     * Returns the absolute minimum value of the range that has been set at construction time.
     *
     * @return The absolute minimum value of the range.
     */
    public T getAbsoluteMinValue() {
        return getObject().getAbsoluteMinValue();
    }

    /**
     * Returns the absolute maximum value of the range that has been set at construction time.
     *
     * @return The absolute maximum value of the range.
     */
    public T getAbsoluteMaxValue() {
        return getObject().getAbsoluteMaxValue();
    }

    /**
     * Returns the currently selected min value.
     *
     * @return The currently selected min value.
     */
    public T getSelectedMinValue() {
        return getObject().getSelectedMinValue();
    }

    /**
     * Sets the currently selected minimum value. The widget will be invalidated and redrawn.
     *
     * @param value The Number value to set the minimum value to. Will be clamped to given absolute minimum/maximum range.
     */
    public void setSelectedMinValue(T value) {
        getObject().setSelectedMinValue(value);
    }

    /**
     * Returns the currently selected max value.
     *
     * @return The currently selected max value.
     */
    public T getSelectedMaxValue() {
        return getObject().getSelectedMaxValue();
    }

    /**
     * Sets the currently selected maximum value. The widget will be invalidated and redrawn.
     *
     * @param value The Number value to set the maximum value to. Will be clamped to given absolute minimum/maximum range.
     */
    public void setSelectedMaxValue(T value) {
        getObject().setSelectedMaxValue(value);
    }

/*    public boolean onTouchEvent(MotionEvent event) {
        return getObject().onTouchEvent(event);
    }
*/

    void AddToParent(ViewGroup Parent, @Pixel int left, @Pixel int top, @Pixel int width, @Pixel int height) {
        Parent.addView(getObject(), new BALayout.LayoutParams(left, top, width, height));
    }

    //this method may not be hidden.
    /**
     * This method is to be used by the B4A Designer and should not be called programmatically.
     */
    public void DesignerCreateView(PanelWrapper base, LabelWrapper lw, anywheresoftware.b4a.objects.collections.Map props) {
        ViewGroup vg = (ViewGroup) base.getObject().getParent();
        AddToParent(vg, base.getLeft(), base.getTop(), base.getWidth(), base.getHeight());
        base.RemoveView();
        //set text properties
    }

    public void setLeft(int left)	{
        BALayout.LayoutParams lp = (BALayout.LayoutParams)getObject().getLayoutParams();
        lp.left = left;
        getObject().getParent().requestLayout();
    }

    public int getLeft()	{
        BALayout.LayoutParams lp = (BALayout.LayoutParams)getObject().getLayoutParams();
        return lp.left;
    }

    public void setTop(int top)	{
        BALayout.LayoutParams lp = (BALayout.LayoutParams)getObject().getLayoutParams();
        lp.top = top;
        getObject().getParent().requestLayout();
    }

    public int getTop()	{
        BALayout.LayoutParams lp = (BALayout.LayoutParams)getObject().getLayoutParams();
        return lp.top;
    }

    public void setWidth(int Width)	{
        BALayout.LayoutParams lp = (BALayout.LayoutParams)getObject().getLayoutParams();
        lp.width = Width;
        getObject().getParent().requestLayout();
    }

    public int getWidth()	{
        BALayout.LayoutParams lp = (BALayout.LayoutParams)getObject().getLayoutParams();
        return lp.width;
    }

    public void setHeight(int height)	{
        BALayout.LayoutParams lp = (BALayout.LayoutParams)getObject().getLayoutParams();
        lp.height = height;
        getObject().getParent().requestLayout();
    }

    public int getHeight()	{
        BALayout.LayoutParams lp = (BALayout.LayoutParams)getObject().getLayoutParams();
        return lp.height;
    }
}
